import {
  Loader, Metadata, IEventParam, IDictionary,
} from '@zeedhi/core';
import {
  Form, Select, Button, Date as DateInput,
} from '@zeedhi/common';
import { default as moment } from 'dayjs';

export class Pme01000EmissaoCardSemanaController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');
  
  public modalController = Loader.getInstance('ModalController');
  
  public numeroDaSemanaDe = 0;
  
  public numeroDaSemanaAte = 0;

  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }

  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');

    if (this.formIsValid(form)) {
      const row: IDictionary = form.value;
      
      this.setParams(row);
      if (!row.CDFILCENTPME) this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'error');
      else {
        row.reportFormat = component.name;
        this.utilities.activeLoadingButton(component);
        this.reportController.generateReport('report/emissaoCardSemana', row, row.CD_REPORT_CODE).finally(() => {
          this.utilities.disableLoadingButton(component);
        });
      }
    }
  }
  
  private setParams(row: IDictionary) {
    row.CDTIPOCARD = row.TIPOCARD.CDTIPOCARD;
    row.NMTIPOCARD = row.TIPOCARD.NMTIPOCARD;
    row.NMFILIAGRE = row.UNIDADE ? row.UNIDADE.NMFILIAL : null;
    row.CDFILIAL = row.UNIDADE ? row.UNIDADE.CDFILIAL : this.appController.getCdCentral();
    row.CDSERVFILI = row.SERVICO ? row.SERVICO : null;
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
  }

  private formIsValid(form: Form) {
    let valid = false;
    const row: IDictionary = form.value;
    if (form.validate()) {
      if (this.utilities.dateIsValid(row.DTDE, row.DTATE)) {
        valid = true;
      } else this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'error');
    }
    return valid;
  }
  
  public setaDataInicioEFimDaSemana() {
    this.utilities.getProcessDate('').then((dataProcessamento: string) => {
      const componenteDataInicial: DateInput = Metadata.getInstance('DTDE');
      const componenteDataFinal: DateInput = Metadata.getInstance('DTATE');
      
      const data: IDictionary = dataProcessamento.split('/').reverse();
      const diaDaDataProcessada = new Date(data[0], data[1] - 1, data[2]);
      const dia = diaDaDataProcessada.getDay();
      const diferencasDosDias = ((dia + 6) % 7);
      const valorParaSubtrair = (diferencasDosDias === 0) ? 7 : diferencasDosDias;
      const ultimaSegunda = moment(dataProcessamento, 'DD/MM/YYYY').subtract(valorParaSubtrair, 'd').format('DD/MM/YYYY');
      const primeiroDomingo = moment(ultimaSegunda, 'DD/MM/YYYY').add(6, 'd').format('DD/MM/YYYY');
      
      componenteDataInicial.value = ultimaSegunda;
      componenteDataFinal.value = primeiroDomingo;
      
      this.numeroDaSemanaDe = this.setaNumeroDaSemanaPelaData(diaDaDataProcessada);
      this.numeroDaSemanaAte = this.numeroDaSemanaDe;
    });
  }
  
  public criaData(value: string) {
    const data = value;
    const diaMesAno: IDictionary = data.split('/').reverse();
    
    return new Date(diaMesAno[0], diaMesAno[1] - 1, diaMesAno[2]);
  }
  
  public alteraNumeroSemana({ component }: IDictionary) {
    const nomeComponente = component.name;
    const dataInstanciada = this.criaData(component.value);
    
    if (nomeComponente === 'DTDE') this.numeroDaSemanaDe = this.setaNumeroDaSemanaPelaData(dataInstanciada);
    else this.numeroDaSemanaAte = this.setaNumeroDaSemanaPelaData(dataInstanciada);
  }
  
  private setaNumeroDaSemanaPelaData(data: Date) {
    const primeiroDiaDoAno = new Date(data.getFullYear(), 0, 1);
    const quantosDiasFaltamPraAcabarOAno = (data.valueOf() - primeiroDiaDoAno.valueOf()) / 86400000;
    
    return Math.ceil((quantosDiasFaltamPraAcabarOAno + primeiroDiaDoAno.getDay() + 1) / 7);
  }
  
  public alteraValorDoPeriodo({ component, event }: IDictionary) {
    const componenteDataInicial: DateInput = Metadata.getInstance('DTDE');
    const componenteDataFinal: DateInput = Metadata.getInstance('DTATE');
    const dataInicial = componenteDataInicial.value;
    const dataFinal = componenteDataFinal.value;
    const iconClicked = (event as IDictionary).currentTarget.ariaLabel;
    const nomeComponente = component.name;
    
    if (iconClicked === 'append icon') {
      if (nomeComponente === 'NRSEMANADE') {
        componenteDataInicial.value = moment(dataInicial, 'DD/MM/YYYY').add(7, 'd').format('DD/MM/YYYY');
        const data = this.criaData(componenteDataInicial.value);
        
        this.numeroDaSemanaDe = this.setaNumeroDaSemanaPelaData(data);
      } else {
        componenteDataFinal.value = moment(dataFinal, 'DD/MM/YYYY').add(7, 'd').format('DD/MM/YYYY');
        const data = this.criaData(componenteDataFinal.value);
        
        this.numeroDaSemanaAte = this.setaNumeroDaSemanaPelaData(data);
      }
    } else if (iconClicked !== 'append icon') {
      if (nomeComponente === 'NRSEMANADE') {
        componenteDataInicial.value = moment(dataInicial, 'DD/MM/YYYY').subtract(7, 'd').format('DD/MM/YYYY');
        const data = this.criaData(componenteDataInicial.value);
        
        this.numeroDaSemanaDe = this.setaNumeroDaSemanaPelaData(data);
      } else {
        componenteDataFinal.value = moment(dataFinal, 'DD/MM/YYYY').subtract(7, 'd').format('DD/MM/YYYY');
        const data = this.criaData(componenteDataFinal.value);
        
        this.numeroDaSemanaAte = this.setaNumeroDaSemanaPelaData(data);
      }
    }
  }

  public openModalSearch() {
    this.modalController.openModal();
    const tipoCard = Metadata.getInstance<Select>('TIPOCARD').value;
    this.utilities.setCdTipoCard(tipoCard.CDTIPOCARD);
  }

  public async createPeriodoModal() {
    this.modalController.createModal('/modals/gridPeriodo');
  }

  public destroyPeriodoModal() {
    this.modalController.destroyModal();
  }
}
